local assets =
{
    Asset("ANIM", "anim/firetrap.zip"),
    --Asset("ANIM", "anim/trap_teeth_maxwell.zip"),
	Asset("MINIMAP_IMAGE", "toothtrap"),
	Asset("ATLAS", "images/inventoryimages/firetrap.xml"),
}

local function onfinished_normal(inst)
    inst:RemoveComponent("inventoryitem")
    inst:RemoveComponent("mine")
    inst.persists = false
    inst.AnimState:PushAnimation("used", false)
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
    inst:DoTaskInTime(3, inst.Remove)
end

local function OnExplode(inst, target)
    inst.AnimState:PlayAnimation("trap")
	
	--Burn the target
	
	if target.components.health then 
		target.components.health:DoFireDamage(TUNING.FIRETRAP_DAMAGE, inst, true)
	end
	
	if target.components.burnable then
		target.components.burnable:Ignite(true, inst)
	end
	
	--Thow fire everywhere!
	if inst.components.lootdropper == nil then
		inst:AddComponent("lootdropper")
	end
	for i = 1, TUNING.FIRETRAP_FIRECOUNT do
		inst.components.lootdropper:SpawnLootPrefab("firetrap_fire", inst:GetPosition())
	end	
	
    --[[
	if target then
        inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_trigger")
        target.components.combat:GetAttacked(inst, TUNING.TRAP_TEETH_DAMAGE)
    end
	]]
    if inst.components.finiteuses then
        inst.components.finiteuses:Use(1)
    end
end

local function OnReset(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_reset")
    inst.AnimState:PlayAnimation("reset")
    inst.AnimState:PushAnimation("idle", false)
end

local function OnResetMax(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/trap_teeth_reset")
    inst.AnimState:PlayAnimation("idle")
    --inst.AnimState:PushAnimation("idle", false)
end

local function SetSprung(inst)
    inst.AnimState:PlayAnimation("trap_idle")
end

local function SetInactive(inst)
    inst.AnimState:PlayAnimation("inactive")
end

local function OnDropped(inst)
    inst.components.mine:Deactivate()
end

local function ondeploy(inst, pt, deployer)
    inst.components.mine:Reset()
    inst.Physics:Teleport(pt:Get())
end

local function fn(bank, build, isinventoryitem)
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()

    MakeInventoryPhysics(inst)

    inst.MiniMapEntity:SetIcon("toothtrap.png")

    inst.AnimState:SetBank("trap_teeth")
    inst.AnimState:SetBuild("firetrap")
    inst.AnimState:PlayAnimation("idle")

    inst:AddTag("trap")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("inspectable")

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.nobounce = true
    inst.components.inventoryitem:SetOnDroppedFn(OnDropped)
	inst.components.inventoryitem.imagename = "firetrap"
	inst.components.inventoryitem.atlasname = "images/inventoryimages/firetrap.xml"

    inst:AddComponent("mine")
    inst.components.mine:SetRadius(TUNING.TRAP_TEETH_RADIUS)
    inst.components.mine:SetAlignment("player")
    inst.components.mine:SetOnExplodeFn(OnExplode)
    inst.components.mine:SetOnResetFn(OnReset)
    inst.components.mine:SetOnSprungFn(SetSprung)
    inst.components.mine:SetOnDeactivateFn(SetInactive)
    --inst.components.mine:StartTesting()

    inst:AddComponent("finiteuses")
    inst.components.finiteuses:SetMaxUses(TUNING.FIRETRAP_USES)
    inst.components.finiteuses:SetUses(TUNING.FIRETRAP_USES)
    inst.components.finiteuses:SetOnFinished(onfinished_normal)

    inst:AddComponent("deployable")
    inst.components.deployable.ondeploy = ondeploy
    inst.components.deployable:SetDeploySpacing(DEPLOYSPACING.LESS)

    inst.components.mine:Deactivate()
    return inst
end

local function fireburn(inst)
	local x, y, z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x, y, z, TUNING.FIRETRAP_FIRE_RANGE)
	
	print("fireburnfn call")

	for i, v in ipairs(ents) do
		if v.components.health then
			v.components.health:DoFireDamage(TUNING.FIRETRAP_FIRE_DAMAGE, inst, true)
			--v.components.health:DoDelta(-TUNING.FIRETRAP_FIRE_DAMAGE, false, "fire")
            --v.components.health.inst:PushEvent("firedamage")   
		end
	end
end

local function fnfiredrop()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddNetwork()

    MakeInventoryPhysics(inst)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    MakeMediumBurnable(inst, 6 + math.random() * 6)
    MakeMediumPropagator(inst)

    --Remove the default handlers that toggle persists flag
    inst.components.burnable:SetOnIgniteFn(nil)
    inst.components.burnable:SetOnExtinguishFn(inst.Remove)
	inst.components.burnable.onburnt = nil
    inst.components.burnable:Ignite()
	
	fireburn(inst)
	inst:DoPeriodicTask(1, fireburn)

    return inst
end

return Prefab("firetrap", fn, assets),
    MakePlacer("firetrap_placer", "trap_teeth", "firetrap", "idle"),
	Prefab("firetrap_fire", fnfiredrop, assets)
