local assets =
{
    Asset("ANIM", "anim/lighter.zip"),
    Asset("ANIM", "anim/swap_lighter.zip"),
	
	Asset("ANIM", "anim/campfire_fire.zip"),
	Asset("ANIM", "anim/swap_lighter_shadow.zip"),
	Asset("ATLAS", "images/inventoryimages/lighter_shadow.xml"),
	 
    --Asset("SOUND", "sound/common.fsb"),
}

local prefabs =
{
    "lighterfire",
	"nightlight_flame",
	"lighterfire_shadow",
}

local function createFire(inst)
	local owner = inst.components.inventoryitem.owner

	if owner.components.sanity:GetPercent() < TUNING.LIGHTER_SHADOW_SANITY_THRESHOLD then
		--We don't have enough sanity to maintain the lighter
		inst.components.burnable:Extinguish()
		
		if inst.fire then
			inst.fire:Remove()
			inst.fire = nil
		end
		
		inst.components.equippable.dapperness = 0
	else
		inst.components.burnable:Ignite()
	
		--We have enough sanity, activate the lighter
		if inst.fire == nil then
			inst.fire = SpawnPrefab("lighterfire_shadow")
			inst.fire.entity:AddFollower()
			inst.fire.Follower:FollowSymbol(owner.GUID, "swap_object", 0, 0, 0)
		end
		
		inst.components.equippable.dapperness = TUNING.LIGHTER_SHADOW_DAPPERNESS
    end
end

local function onequip(inst, owner)
    inst.components.burnable:Ignite()
    owner.AnimState:OverrideSymbol("swap_object", "swap_lighter_shadow", "swap_lighter_shadow")
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")

    --owner.SoundEmitter:PlaySound("dontstarve/wilson/lighter_on")
	
	if inst.fire == nil then
		inst.fire = SpawnPrefab("lighterfire_shadow")
		--inst.fire.Transform:SetScale(.125, .125, .125)
		inst.fire.entity:AddFollower()
		--inst.fire.Follower:FollowSymbol(owner.GUID, "swap_object", 56, -40, 0)
		inst.fire.Follower:FollowSymbol(owner.GUID, "swap_object", 0, 0, 0)
	end
	
	--[[
	if inst.fireUpdateTask then
		inst.fireUpdateTask:Cancel()
		inst.fireUpdateTask = nil
	end

    createFire(inst, owner)
	inst.fireUpdateTask = inst:DoPeriodicTask(1, createFire)
	]]
end

local function onunequip(inst,owner)
    if inst.fire ~= nil then
        inst.fire:Remove()
        inst.fire = nil
    end
	
	--[[
	if inst.fireUpdateTask then
		inst.fireUpdateTask:Cancel()
		inst.fireUpdateTask = nil
	end
	]]

    inst.components.burnable:Extinguish()
    owner.AnimState:Hide("ARM_carry")
    owner.AnimState:Show("ARM_normal")
    owner.SoundEmitter:PlaySound("dontstarve/wilson/lighter_off")
end

local function onpocket(inst, owner)
    inst.components.burnable:Extinguish()
end

local function onattack(weapon, attacker, target)
    if target ~= nil and target.components.burnable ~= nil and math.random() < TUNING.LIGHTER_ATTACK_IGNITE_PERCENT * target.components.burnable.flammability then
        target.components.burnable:Ignite(nil, attacker)
    end
end

--[[
local function oncook(inst, product, chef)
    if not chef:HasTag("expertchef") then
        --burn
        if chef.components.health ~= nil then
            chef.components.health:DoFireDamage(5, inst, true)
            chef:PushEvent("burnt")
        end
        if inst.components.fueled ~= nil then
            inst.components.fueled:DoDelta(-.05 * inst.components.fueled.maxfuel)
        end
    elseif inst.components.fueled ~= nil then
        inst.components.fueled:DoDelta(-.01 * inst.components.fueled.maxfuel)
    end
end
]]

local function ligher_forcedrop(inst, owner)
	if not owner:HasTag("pyromaniac") then
		--lighter burns, drop it
		if owner.components.health ~= nil then
            owner.components.health:DoFireDamage(5, inst, true)
            owner:PushEvent("burnt")
        end
		
		owner:DoTaskInTime(0, function()
			owner.components.inventory:DropItem(inst, nil, true)
		end)
	end
end

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()

    MakeInventoryPhysics(inst)
	
	--I'm not crazy I promise. I want it to look like a tiny fire.
	inst.AnimState:SetBank("campfire_fire")
    inst.AnimState:SetBuild("campfire_fire")
    inst.AnimState:PlayAnimation("level2", true)
    inst.AnimState:SetMultColour(0, 0, 0, .6)

    inst.MiniMapEntity:SetIcon("lighter_shadow.tex")

    --inst:AddTag("dangerouscooker")
    inst:AddTag("wildfireprotected")
	inst:AddTag("willowsanityexcluded")

    --lighter (from lighter component) added to pristine state for optimization
    inst:AddTag("lighter")

    --cooker (from cooker component) added to pristine state for optimization
    --inst:AddTag("cooker")

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(TUNING.LIGHTER_DAMAGE)
    inst.components.weapon:SetOnAttack(onattack)

    -----------------------------------
    inst:AddComponent("lighter")
    -----------------------------------
    inst:AddComponent("inventoryitem")
	inst.components.inventoryitem.imagename = "lighter_shadow"
	inst.components.inventoryitem.atlasname = "images/inventoryimages/lighter_shadow.xml"
	inst.components.inventoryitem:SetOnPutInInventoryFn(ligher_forcedrop)
    -----------------------------------
    --inst:AddComponent("cooker")
    --inst.components.cooker.oncookfn = oncook
    -----------------------------------

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnPocket(onpocket)
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
	
	inst.components.equippable.dapperness = TUNING.LIGHTER_SHADOW_DAPPERNESS
	inst.components.equippable.walkspeedmult = TUNING.LIGHTER_SHADOW_SPEEDMULT

    -----------------------------------

    inst:AddComponent("inspectable")

    -----------------------------------

    inst:AddComponent("burnable")
    inst.components.burnable.canlight = false
    inst.components.burnable.fxprefab = nil

    MakeHauntableLaunch(inst)

    return inst
end

return Prefab("lighter_shadow", fn, assets, prefabs)
